Index: fluid/fluid.cxx
===================================================================
--- fluid/fluid.cxx	(revision 11366)
+++ fluid/fluid.cxx	(working copy)
@@ -167,8 +167,8 @@
   pos.set("visible", (int)(w->shown() && w->visible()));
 }
 
-Fl_Window *main_window;
-Fl_Menu_Bar *main_menubar;
+Fl_Window *main_window = 0;
+Fl_Menu_Bar *main_menubar = 0;
 
 static char* cutfname(int which = 0) {
   static char name[2][FL_PATH_MAX];
@@ -1633,6 +1633,8 @@
 
   modflag = mf;
 
+  if ( batch_mode ) return;	// avoid any GUI code in batch mode
+
   if (main_window) {
     if (!filename) basename = "Untitled.fl";
     else if ((basename = strrchr(filename, '/')) != NULL) basename ++;
@@ -1711,8 +1713,62 @@
 }
 #endif
 
+// Handle -c/-s/-u batch mode flags, or return if none.
+//    There should be *NO GUI CODE* in here; pure console app from here on.
+//
+void handle_csu_flags(int argc,char **argv) {
+  char *filename = 0;
+  for ( int i=1; i<argc; i++ ) {
+    if (argv[i][0] == '-' ) {
+      for ( int ii=1; argv[i][ii]; ii++ ) {
+        switch (argv[i][ii]) {
+	  case 'c': { ++compile_file; ++batch_mode; break; }	// -c
+	  case 's': { ++compile_strings; break; }		// -s, -cs
+	}
+      }
+    } else {
+      if ( compile_file && !filename ) {
+        filename = argv[i];
+        set_filename(filename);
+	continue;
+      }
+    }
+  }
+  if ( !compile_file ) return;
 
+  // Definitely in batch mode, handle it..
+  if ( !filename ) {
+    fprintf(stderr, "fluid: -c expected filename argument\n");
+    exit(1);
+  }
+
+  undo_suspend();
+  if (filename && !read_file(filename, 0)) {
+    fprintf(stderr, "fluid: %s : %s\n", filename, strerror(errno));
+    exit(1);
+  }
+  undo_resume();
+
+  if (update_file) {		// fluid -u
+    write_file(filename,0);
+    exit(0);
+  }
+
+  if (compile_strings) {
+    write_strings_cb(0,0);
+  }
+
+  write_cb(0,0);
+  exit(0);
+}
+
 int main(int argc,char **argv) {
+  // NO CODE ABOVE THIS LINE
+  //    Leave batch mode handling at top of main()
+  //    to avoid ANY gui code from being invoked during batch mode.
+  //
+  handle_csu_flags(argc,argv);		// fluid -c[su] file.fl
+
   int i = 1;
   
   if (!Fl::args(argc,argv,i,arg) || i < argc-1) {
@@ -1756,44 +1812,27 @@
 
 
   if (c) set_filename(c);
-  if (!batch_mode) {
 #ifdef __APPLE__
-    fl_open_callback(apple_open_cb);
+  fl_open_callback(apple_open_cb);
 #endif // __APPLE__
-    Fl::visual((Fl_Mode)(FL_DOUBLE|FL_INDEX));
-    Fl_File_Icon::load_system_icons();
-    main_window->callback(exit_cb);
-    position_window(main_window,"main_window_pos", 1, 10, 30, WINWIDTH, WINHEIGHT );
-    main_window->show(argc,argv);
-    toggle_widgetbin_cb(0,0);
-    toggle_sourceview_cb(0,0);
-    if (!c && openlast_button->value() && absolute_history[0][0]) {
-      // Open previous file when no file specified...
-      open_history_cb(0, absolute_history[0]);
-    }
+  Fl::visual((Fl_Mode)(FL_DOUBLE|FL_INDEX));
+  Fl_File_Icon::load_system_icons();
+  main_window->callback(exit_cb);
+  position_window(main_window,"main_window_pos", 1, 10, 30, WINWIDTH, WINHEIGHT );
+  main_window->show(argc,argv);
+  toggle_widgetbin_cb(0,0);
+  toggle_sourceview_cb(0,0);
+  if (!c && openlast_button->value() && absolute_history[0][0]) {
+    // Open previous file when no file specified...
+    open_history_cb(0, absolute_history[0]);
   }
+
   undo_suspend();
   if (c && !read_file(c,0)) {
-    if (batch_mode) {
-      fprintf(stderr,"%s : %s\n", c, strerror(errno));
-      exit(1);
-    }
     fl_message("Can't read %s: %s", c, strerror(errno));
   }
   undo_resume();
 
-  if (update_file) {		// fluid -u
-    write_file(c,0);
-    if (!compile_file)
-      exit(0);
-  }
-
-  if (compile_file) {		// fluid -c[s]
-    if (compile_strings)
-      write_strings_cb(0,0);
-    write_cb(0,0);
-    exit(0);
-  }
   set_modflag(0);
   undo_clear();
 #ifndef WIN32
